# Generated by Django 3.2.4 on 2021-07-02 11:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0003_auto_20210701_1508'),
    ]

    operations = [
        migrations.CreateModel(
            name='Serial',
            fields=[
                ('uuid', models.UUIDField(primary_key=True, serialize=False)),
            ],
        ),
    ]
